@@ -12,9 +12,11 @@ This module defines Unicode character categories and blocks.
 """
 from sys import maxunicode
 from collections.abc import Iterable, MutableSet
+from typing import Iterator, List
 
 from .unicode_categories import RAW_UNICODE_CATEGORIES
-from .codepoints import code_point_order, code_point_repr, iter_code_points, get_code_point_range
+from .codepoints import CodePoint, code_point_order, code_point_repr, \
+    iter_code_points, get_code_point_range
 
 
 class RegexError(Exception):
@@ -25,7 +27,7 @@ class RegexError(Exception):
     """
 
 
-def iterparse_character_subset(s, expand_ranges=False):
+def iterparse_character_subset(s: str, expand_ranges=False) -> Iterator[CodePoint]:
     """
     Parses a regex character subset, generating a sequence of code points
     and code points ranges. An unescaped hyphen (-) that is not at the
@@ -37,7 +39,7 @@ def iterparse_character_subset(s, expand_ranges=False):
     """
     escaped = False
     on_range = False
-    char = None
+    char = ''
     length = len(s)
     subset_index_iterator = iter(range(len(s)))
     for k in subset_index_iterator:
@@ -123,6 +125,7 @@ class UnicodeSubset(MutableSet):
     instance ora a string equivalent of a regex character set.
     """
     __slots__ = '_codepoints',
+    _codepoints: List[CodePoint]
 
     def __init__(self, codepoints=None):
         if not codepoints:
@@ -140,7 +143,7 @@ class UnicodeSubset(MutableSet):
         return self._codepoints
 
     def __repr__(self):
-        return "%s(%r)" % (self.__class__.__name__, str(self))
+        return '%s(%r)' % (self.__class__.__name__, str(self))
 
     def __str__(self):
         return ''.join(code_point_repr(cp) for cp in self._codepoints)
@@ -336,6 +339,10 @@ class UnicodeSubset(MutableSet):
             self.add(cp)
         return self
 
+    def __or__(self, other):
+        obj = self.copy()
+        return obj.__ior__(other)
+
     def __isub__(self, other):
         if not isinstance(other, Iterable):
             return NotImplemented
@@ -361,6 +368,10 @@ class UnicodeSubset(MutableSet):
             self.discard(value)
         return self
 
+    def __and__(self, other):
+        obj = self.copy()
+        return obj.__iand__(other)
+
     def __ixor__(self, other):
         if other is self:
             self.clear()
@@ -377,6 +388,10 @@ class UnicodeSubset(MutableSet):
                 self.add(value)
         return self
 
+    def __xor__(self, other):
+        obj = self.copy()
+        return obj.__ixor__(other)
+
 
 UNICODE_CATEGORIES = {k: UnicodeSubset(v) for k, v in RAW_UNICODE_CATEGORIES.items()}
 
