@@ -7,6 +7,7 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
+from ..exceptions import ElementPathTypeError
 from ..xpath_nodes import NamespaceNode, is_element_node
 from .xpath1_functions import XPath1Parser
 
@@ -15,7 +16,7 @@ axis = XPath1Parser.axis
 
 
 @method('@', bp=80)
-def nud(self):
+def nud_attribute_reference(self):
     self.parser.expected_name(
         '*', '(name)', ':', '{', 'Q{', message="invalid attribute specification")
     self[:] = self.parser.expression(rbp=80),
@@ -24,16 +25,19 @@ def nud(self):
 
 @method('@')
 @method(axis('attribute'))
-def select(self, context=None):
+def select_attribute_reference_or_axis(self, context=None):
     if context is None:
         raise self.missing_context()
 
     for _ in context.iter_attributes():
-        yield from self[0].select(context)
+        try:
+            yield from self[0].select(context)
+        except ElementPathTypeError:
+            pass
 
 
 @method(axis('namespace'))
-def select(self, context=None):
+def select_namespace_axis(self, context=None):
     if context is None:
         raise self.missing_context()
     elif is_element_node(context.item):
@@ -54,7 +58,7 @@ def select(self, context=None):
 
 
 @method(axis('self'))
-def select(self, context=None):
+def select_self_axis(self, context=None):
     if context is None:
         raise self.missing_context()
     else:
@@ -63,7 +67,7 @@ def select(self, context=None):
 
 
 @method(axis('child'))
-def select(self, context=None):
+def select_child_axis(self, context=None):
     if context is None:
         raise self.missing_context()
     else:
@@ -72,7 +76,7 @@ def select(self, context=None):
 
 
 @method(axis('parent', reverse_axis=True))
-def select(self, context=None):
+def select_parent_axis(self, context=None):
     if context is None:
         raise self.missing_context()
     else:
@@ -82,7 +86,7 @@ def select(self, context=None):
 
 @method(axis('following-sibling'))
 @method(axis('preceding-sibling', reverse_axis=True))
-def select(self, context=None):
+def select_sibling_axes(self, context=None):
     if context is None:
         raise self.missing_context()
     else:
@@ -92,7 +96,7 @@ def select(self, context=None):
 
 @method(axis('ancestor', reverse_axis=True))
 @method(axis('ancestor-or-self', reverse_axis=True))
-def select(self, context=None):
+def select_ancestor_axes(self, context=None):
     if context is None:
         raise self.missing_context()
     else:
@@ -102,7 +106,7 @@ def select(self, context=None):
 
 @method(axis('descendant'))
 @method(axis('descendant-or-self'))
-def select(self, context=None):
+def select_descendant_axes(self, context=None):
     if context is None:
         raise self.missing_context()
     else:
@@ -111,7 +115,7 @@ def select(self, context=None):
 
 
 @method(axis('following'))
-def select(self, context=None):
+def select_following_axis(self, context=None):
     if context is None:
         raise self.missing_context()
     else:
@@ -120,12 +124,9 @@ def select(self, context=None):
 
 
 @method(axis('preceding', reverse_axis=True))
-def select(self, context=None):
+def select_preceding_axis(self, context=None):
     if context is None:
         raise self.missing_context()
     elif is_element_node(context.item):
         for _ in context.iter_preceding():
             yield from self[0].select(context)
-
-
-register = XPath1Parser.register('(end)')
