@@ -1,5 +1,5 @@
 #
-# Copyright (c), 2018-2020, SISSA (International School for Advanced Studies).
+# Copyright (c), 2018-2021, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -16,9 +16,7 @@ from ..datatypes import xsd10_atomic_types, xsd11_atomic_types, GregorianDay, \
     GregorianMonth, GregorianMonthDay, GregorianYear10, GregorianYear, \
     GregorianYearMonth10, GregorianYearMonth, Duration, DayTimeDuration, \
     YearMonthDuration, Date10, Date, DateTime10, DateTime, DateTimeStamp, \
-    Time, UntypedAtomic, QName, HexBinary, Base64Binary, BooleanProxy, ATOMIC_VALUES
-from ..xpath_token import XPathToken
-from ..xpath_context import XPathSchemaContext
+    Time, UntypedAtomic, QName, HexBinary, Base64Binary, BooleanProxy
 from .xpath2_functions import XPath2Parser
 
 
@@ -40,7 +38,7 @@ constructor = XPath2Parser.constructor
 @constructor('IDREF')
 @constructor('ENTITY')
 @constructor('anyURI')
-def cast(self, value):
+def cast_string_based_types(self, value):
     try:
         return xsd10_atomic_types[self.symbol](value)
     except ValueError as err:
@@ -52,7 +50,7 @@ def cast(self, value):
 @constructor('decimal')
 @constructor('double')
 @constructor('float')
-def cast(self, value):
+def cast_numeric_types(self, value):
     try:
         if self.parser.xsd_version == '1.0':
             return xsd10_atomic_types[self.symbol](value)
@@ -76,7 +74,7 @@ def cast(self, value):
 @constructor('unsignedInt')
 @constructor('unsignedShort')
 @constructor('unsignedByte')
-def cast(self, value):
+def cast_integer_types(self, value):
     try:
         return xsd10_atomic_types[self.symbol](value)
     except ValueError:
@@ -85,15 +83,13 @@ def cast(self, value):
             raise self.error('FORG0001', msg) from None
         raise self.error('FOCA0002', msg) from None
     except OverflowError as err:
-        if isinstance(value, (str, bytes, UntypedAtomic)):
-            raise self.error('FORG0001', err) from None
         raise self.error('FOCA0002', err) from None
 
 
 ###
 # Constructors for datetime XSD types
 @constructor('date')
-def cast(self, value):
+def cast_date_type(self, value):
     cls = Date if self.parser.xsd_version == '1.1' else Date10
     if isinstance(value, cls):
         return value
@@ -111,7 +107,7 @@ def cast(self, value):
 
 
 @constructor('gDay')
-def cast(self, value):
+def cast_gregorian_day_type(self, value):
     if isinstance(value, GregorianDay):
         return value
 
@@ -126,7 +122,7 @@ def cast(self, value):
 
 
 @constructor('gMonth')
-def cast(self, value):
+def cast_gregorian_month_type(self, value):
     if isinstance(value, GregorianMonth):
         return value
 
@@ -141,7 +137,7 @@ def cast(self, value):
 
 
 @constructor('gMonthDay')
-def cast(self, value):
+def cast_gregorian_month_day_type(self, value):
     if isinstance(value, GregorianMonthDay):
         return value
 
@@ -156,7 +152,7 @@ def cast(self, value):
 
 
 @constructor('gYear')
-def cast(self, value):
+def cast_gregorian_year_type(self, value):
     cls = GregorianYear if self.parser.xsd_version == '1.1' else GregorianYear10
     if isinstance(value, cls):
         return value
@@ -174,7 +170,7 @@ def cast(self, value):
 
 
 @constructor('gYearMonth')
-def cast(self, value):
+def cast_gregorian_year_month_type(self, value):
     cls = GregorianYearMonth \
         if self.parser.xsd_version == '1.1' else GregorianYearMonth10
     if isinstance(value, cls):
@@ -193,7 +189,7 @@ def cast(self, value):
 
 
 @constructor('time')
-def cast(self, value):
+def cast_time_type(self, value):
     if isinstance(value, Time):
         return value
 
@@ -215,7 +211,7 @@ def cast(self, value):
 @method('gYear')
 @method('gYearMonth')
 @method('time')
-def evaluate(self, context=None):
+def evaluate_other_datetime_types(self, context=None):
     arg = self.data_value(self.get_argument(context))
     if arg is None:
         return []
@@ -231,7 +227,7 @@ def evaluate(self, context=None):
 ###
 # Constructors for time durations XSD types
 @constructor('duration')
-def cast(self, value):
+def cast_duration_type(self, value):
     if isinstance(value, Duration):
         return value
 
@@ -246,7 +242,7 @@ def cast(self, value):
 
 
 @constructor('yearMonthDuration')
-def cast(self, value):
+def cast_year_month_duration_type(self, value):
     if isinstance(value, YearMonthDuration):
         return value
     elif isinstance(value, Duration):
@@ -263,7 +259,7 @@ def cast(self, value):
 
 
 @constructor('dayTimeDuration')
-def cast(self, value):
+def cast_day_time_duration_type(self, value):
     if isinstance(value, DayTimeDuration):
         return value
     elif isinstance(value, Duration):
@@ -276,65 +272,77 @@ def cast(self, value):
     except OverflowError as err:
         raise self.error('FODT0002', err) from None
     except ValueError as err:
-        raise self.error('FORG0001', err)
+        raise self.error('FORG0001', err) from None
 
 
 @constructor('dateTimeStamp')
-def cast(self, value):
+def cast_datetime_stamp_type(self, value):
     if isinstance(value, DateTimeStamp):
         return value
     elif isinstance(value, DateTime10):
         value = str(value)
 
     try:
+        if isinstance(value, UntypedAtomic):
+            return DateTimeStamp.fromstring(value.value)
         return DateTimeStamp.fromstring(value)
     except ValueError as err:
-        raise self.error('FORG0001', err)
+        raise self.error('FORG0001', err) from None
 
 
 @method('dateTimeStamp')
-def evaluate(self, context=None):
+def evaluate_datetime_stamp_type(self, context=None):
     arg = self.data_value(self.get_argument(context))
     if arg is None:
         return []
 
+    if isinstance(arg, UntypedAtomic):
+        return self.cast(arg.value)
+    return self.cast(str(arg))
+
+
+@method('dateTimeStamp')
+def nud_datetime_stamp_type(self):
+    if self.parser.xsd_version == '1.0':
+        raise self.wrong_syntax("xs:dateTimeStamp is not recognized unless XSD 1.1 is enabled")
+
     try:
-        if isinstance(arg, UntypedAtomic):
-            return self.cast(arg.value)
-        return self.cast(str(arg))
-    except TypeError as err:
-        raise self.error('FORG0006', err) from None
+        self.parser.advance('(')
+        self[0:] = self.parser.expression(5),
+        if self.parser.next_token.symbol == ',':
+            raise self.wrong_nargs('Too many arguments: expected at most 1 argument')
+        self.parser.advance(')')
+        self.value = None
+    except SyntaxError:
+        raise self.error('XPST0017') from None
+    return self
 
 
 ###
 # Constructors for binary XSD types
 @constructor('base64Binary')
-def cast(self, value):
+def cast_base64_binary_type(self, value):
     try:
         return Base64Binary(value)
     except ValueError as err:
         raise self.error('FORG0001', err) from None
     except TypeError as err:
-        if isinstance(value, str):
-            raise self.error('FORG0006', err) from None
         raise self.error('XPTY0004', err) from None
 
 
 @constructor('hexBinary')
-def cast(self, value):
+def cast_hex_binary_type(self, value):
     try:
         return HexBinary(value)
     except ValueError as err:
         raise self.error('FORG0001', err) from None
     except TypeError as err:
-        if isinstance(value, str):
-            raise self.error('FORG0006', err) from None
         raise self.error('XPTY0004', err) from None
 
 
 @method('base64Binary')
 @method('hexBinary')
-def evaluate(self, context=None):
+def evaluate_binary_types(self, context=None):
     arg = self.data_value(self.get_argument(context))
     if arg is None:
         return []
@@ -344,17 +352,15 @@ def evaluate(self, context=None):
     except ElementPathError as err:
         err.token = self
         raise
-    except UnicodeEncodeError as err:
-        raise self.error('FORG0001', err) from None
 
 
 @constructor('NOTATION')
-def cast(self, value):
-    return value
+def cast_notation_type(self, value):
+    raise NotImplementedError("No value is castable to xs:NOTATION")
 
 
 @method('NOTATION')
-def nud(self):
+def nud_notation_type(self):
     self.parser.advance('(')
     if self.parser.next_token.symbol == ')':
         raise self.error('XPST0017', 'expected exactly one argument')
@@ -375,20 +381,18 @@ def nud(self):
 unregister('boolean')
 
 
-@constructor('boolean', bp=90, label=('function', 'constructor function'))
-def cast(self, value):
+@constructor('boolean', label=('function', 'constructor function'))
+def cast_boolean_type(self, value):
     try:
         return BooleanProxy(value)
     except ValueError as err:
         raise self.error('FORG0001', err) from None
     except TypeError as err:
-        if isinstance(value, str):
-            raise self.error('FORG0006', err) from None
         raise self.error('XPTY0004', err) from None
 
 
 @method('boolean')
-def nud(self):
+def nud_boolean_type_and_function(self):
     self.parser.advance('(')
     if self.parser.next_token.symbol == ')':
         raise self.wrong_nargs('Too few arguments: expected at least 1 argument')
@@ -401,7 +405,7 @@ def nud(self):
 
 
 @method('boolean')
-def evaluate(self, context=None):
+def evaluate_boolean_type_and_function(self, context=None):
     if self.label == 'function':
         return self.boolean_value([x for x in self[0].select(context)])
 
@@ -420,12 +424,15 @@ def evaluate(self, context=None):
 ###
 # Case 2: In XPath 2.0 the 'string' keyword is used both for fn:string() and xs:string().
 unregister('string')
-register('string', lbp=90, rbp=90, label=('function', 'constructor function'),  # pragma: no cover
-         pattern=r'\bstring(?=\s*\(|\s*\(\:.*\:\)\()', cast=XPathToken.string_value)
+
+
+@constructor('string', label=('function', 'constructor function'))
+def cast_string_type(self, value):
+    return self.string_value(value)
 
 
 @method('string')
-def nud(self):
+def nud_string_type_and_function(self):
     try:
         self.parser.advance('(')
         if self.label != 'function' or self.parser.next_token.symbol != ')':
@@ -440,7 +447,7 @@ def nud(self):
 
 
 @method('string')
-def evaluate(self, context=None):
+def evaluate_string_type_and_function(self, context=None):
     if self.label == 'function':
         if not self:
             if context is None:
@@ -459,8 +466,9 @@ def evaluate(self, context=None):
 # In those cases the label at parse time is set by the nud method, in dependence
 # of the number of args.
 #
-@constructor('QName', bp=90, label=('function', 'constructor function'))
-def cast(self, value):
+@constructor('QName', bp=90, label=('function', 'constructor function'),
+             sequence_types=('xs:string?', 'xs:string', 'xs:QName'))
+def cast_qname_type(self, value):
     if isinstance(value, QName):
         return value
     elif isinstance(value, UntypedAtomic):
@@ -471,8 +479,9 @@ def cast(self, value):
         raise self.error('XPTY0004', 'the argument has an invalid type %r' % type(value))
 
 
-@constructor('dateTime', bp=90, label=('function', 'constructor function'))
-def cast(self, value):
+@constructor('dateTime', bp=90, label=('function', 'constructor function'),
+             sequence_types=('xs:date?', 'xs:time?', 'xs:dateTime?'))
+def cast_datetime_type(self, value):
     cls = DateTime if self.parser.xsd_version == '1.1' else DateTime10
     if isinstance(value, cls):
         return value
@@ -491,7 +500,7 @@ def cast(self, value):
 
 @method('QName')
 @method('dateTime')
-def nud(self):
+def nud_qname_and_datetime(self):
     try:
         self.parser.advance('(')
         self[0:] = self.parser.expression(5),
@@ -513,7 +522,7 @@ def nud(self):
 
 
 @method('QName')
-def evaluate(self, context=None):
+def evaluate_qname_type_and_function(self, context=None):
     if self.label == 'constructor function':
         arg = self.data_value(self.get_argument(context))
         return [] if arg is None else self.cast(arg)
@@ -525,13 +534,11 @@ def evaluate(self, context=None):
         except TypeError as err:
             raise self.error('XPTY0004', err)
         except ValueError as err:
-            if isinstance(context, XPathSchemaContext):
-                return ATOMIC_VALUES['QName']
             raise self.error('FOCA0002', err)
 
 
 @method('dateTime')
-def evaluate(self, context=None):
+def evaluate_datetime_type_and_function(self, context=None):
     if self.label == 'constructor function':
         arg = self.data_value(self.get_argument(context))
         if arg is None:
@@ -563,12 +570,12 @@ def evaluate(self, context=None):
 
 
 @constructor('untypedAtomic')
-def cast(self, value):
+def cast_untyped_atomic(self, value):
     return UntypedAtomic(value)
 
 
 @method('untypedAtomic')
-def evaluate(self, context=None):
+def evaluate_untyped_atomic(self, context=None):
     arg = self.data_value(self.get_argument(context))
     if arg is None:
         return []
