@@ -9,18 +9,29 @@
 #
 import datetime
 import importlib
-import xml.etree.ElementTree as ElementTree
+from copy import copy
 from functools import lru_cache
 from itertools import chain
+from types import ModuleType
+from typing import TYPE_CHECKING, cast, Dict, Any, List, Iterable, Iterator, \
+    Optional, Sequence, Union, Callable
 
 from .exceptions import ElementPathTypeError
-from .datatypes import Timezone
+from .datatypes import AnyAtomicType, Timezone
 from .xpath_nodes import TypedElement, AttributeNode, TextNode, TypedAttribute, \
     etree_iter_nodes, is_etree_element, is_element_node, is_document_node, \
-    is_schema_node, is_lxml_etree_element, is_lxml_document_node
+    is_schema_node, is_lxml_etree_element, is_lxml_document_node, XPathNode, \
+    ElementNode, DocumentNode
+
+if TYPE_CHECKING:
+    from .xpath_token import XPathToken, XPathAxis
+
+ContextRootType = Union[ElementNode, DocumentNode]
+ContextItemType = Optional[Union[ElementNode, DocumentNode, XPathNode, AnyAtomicType]]
+IterableNodeType = Union[ElementNode, DocumentNode, AttributeNode, TextNode]
 
 
-class XPathContext(object):
+class XPathContext:
     """
     The XPath dynamic context. The static context is provided by the parser.
 
@@ -58,24 +69,36 @@ class XPathContext(object):
     and fn:available-environment-variables.
     """
     _iter_nodes = staticmethod(etree_iter_nodes)
-    _parent_map = None
-    _elem = None
-    _etree = None
+    _parent_map: Optional[Dict[ElementNode, ContextRootType]] = None
+    _etree: Optional[ModuleType] = None
+    root: ContextRootType
+    item: Optional[ContextItemType]
+
+    def __init__(self,
+                 root: ContextRootType,
+                 namespaces: Optional[Dict[str, str]] = None,
+                 item: Optional[ContextItemType] = None,
+                 position: int = 1,
+                 size: int = 1,
+                 axis: Optional[str] = None,
+                 variables: Optional[Dict[str, Any]] = None,
+                 current_dt: Optional[datetime.datetime] = None,
+                 timezone: Optional[Union[str, Timezone]] = None,
+                 documents: Optional[DocumentNode] = None,
+                 collections: Optional[ElementNode] = None,
+                 default_collection: Optional[str] = None,
+                 resource_collections: Optional[Dict[str, List[str]]] = None,
+                 default_resource_collection: Optional[str] = None,
+                 allow_environment: bool = False,
+                 default_language: Optional[str] = None,
+                 default_calendar: Optional[str] = None,
+                 default_place: Optional[str] = None) -> None:
 
-    def __init__(self, root, namespaces=None, item=None, position=1, size=1, axis=None,
-                 variables=None, current_dt=None, timezone=None, documents=None,
-                 collections=None, default_collection=None, resource_collections=None,
-                 default_resource_collection=None, allow_environment=False):
         self.root = root
         self.namespaces = namespaces
 
-        if is_etree_element(root) and not callable(root.tag):
-            if item is None:
-                self.item = self._elem = root
-            elif is_etree_element(item) and not callable(root.tag):
-                self.item = self._elem = item
-            else:
-                self.item, self._elem = item, root
+        if is_element_node(root):
+            self.item = root if item is None else item
         elif is_document_node(root):
             self.item = item
         else:
@@ -103,56 +126,52 @@ class XPathContext(object):
         self.resource_collections = resource_collections
         self.default_resource_collection = default_resource_collection
         self.allow_environment = allow_environment
+        self.default_language = default_language
+        self.default_calendar = default_calendar
+        self.default_place = default_place
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return '%s(root=%r, item=%r)' % (self.__class__.__name__, self.root, self.item)
 
-    def __copy__(self):
-        obj = type(self)(
-            root=self.root,
-            namespaces=self.namespaces,
-            item=self.item,
-            position=self.position,
-            size=self.size,
-            axis=None,
-            variables=self.variables,
-            current_dt=self.current_dt,
-            timezone=self.timezone,
-            documents=self.documents,
-            collections=self.collections,
-            default_collection=self.default_collection,
-        )
-        if self.item is None:
-            obj.item = None
-        obj._elem = self._elem
-        obj._parent_map = self._parent_map
+    def __copy__(self) -> 'XPathContext':
+        obj: XPathContext = object.__new__(self.__class__)
+        obj.__dict__.update(self.__dict__)
+        obj.axis = None
+        obj.variables = {k: v for k, v in self.variables.items()}
         return obj
 
-    def copy(self, clear_axis=True):
+    def copy(self, clear_axis: bool = True) -> 'XPathContext':
+        # Unused, so it could be deprecated in the future.
+        obj: XPathContext = object.__new__(self.__class__)
+        obj.__dict__.update(self.__dict__)
         if clear_axis:
-            return self.__copy__()
-        else:
-            obj = self.__copy__()
-            obj.axis = self.axis
-            return obj
+            obj.axis = None
+        obj.variables = {k: v for k, v in self.variables.items()}
+        return obj
 
     @property
-    def parent_map(self):
+    def parent_map(self) -> Dict[ElementNode, ContextRootType]:
         if self._parent_map is None:
+            self._parent_map: Dict[ElementNode, ContextRootType]
             self._parent_map = {child: elem for elem in self.root.iter() for child in elem}
+            if is_document_node(self.root):
+                self._parent_map[cast(DocumentNode, self.root).getroot()] = self.root
+
         return self._parent_map
 
     @property
-    def etree(self):
+    def etree(self) -> ModuleType:
         if self._etree is None:
+            self._etree: ModuleType
             if is_lxml_etree_element(self.root) or is_lxml_document_node(self.root):
                 self._etree = importlib.import_module('lxml.etree')
             else:
-                self._etree = ElementTree
+                self._etree = importlib.import_module('xml.etree.ElementTree')
         return self._etree
 
     @lru_cache(maxsize=1024)
-    def get_parent(self, elem):
+    def get_parent(self, elem: Union[ElementNode, TypedElement]) \
+            -> Optional[Union[ElementNode, DocumentNode]]:
         """
         Returns the parent element or `None` for root element and for elements
         that are not included in the tree. Uses a LRU cache to minimize parent
@@ -161,29 +180,22 @@ class XPathContext(object):
         if isinstance(elem, TypedElement):
             elem = elem.elem
         if elem is self.root:
-            return
+            return None
 
         try:
-            return self._parent_map[elem]
-        except (KeyError, TypeError):
-            self._parent_map = {child: e for e in self.root.iter() for child in e}
-            if is_document_node(self.root):
-                self._parent_map[self.root.getroot()] = self.root
+            return self.parent_map[elem]
+        except KeyError:
+            return None
 
-            try:
-                return self._parent_map[elem]
-            except KeyError:
-                return
-
-    def get_path(self, item):
+    def get_path(self, item: Any) -> str:
         """Cached path resolver for elements and attributes. Returns absolute paths."""
         path = []
         if isinstance(item, AttributeNode):
-            path.append('@%s' % item.name)
-            item = self._elem
+            path.append(f'@{item.name}')
+            item = item.parent
         elif isinstance(item, TypedAttribute):
-            path.append('@%s' % item.attribute.name)
-            item = self._elem
+            path.append(f'@{item.attribute.name}')
+            item = item.attribute.parent
 
         if item is None:
             return '' if not path else path[0]
@@ -201,21 +213,21 @@ class XPathContext(object):
                 return '/{}'.format('/'.join(reversed(path)))
             item = parent
 
-    def is_principal_node_kind(self):
+    def is_principal_node_kind(self) -> bool:
         if self.axis == 'attribute':
             return isinstance(self.item, (AttributeNode, TypedAttribute))
         else:
             return is_element_node(self.item)
 
-    def iter(self):
+    def iter(self) -> Iterator[IterableNodeType]:
         """Iterates context nodes, including text and attribute nodes."""
         root = self.root
         if is_document_node(root):
             yield root
-            root = root.getroot()
+            root = cast(DocumentNode, root).getroot()
         yield from self._iter_nodes(root, with_attributes=True)
 
-    def iter_results(self, results):
+    def iter_results(self, results: Iterable[Any]) -> Iterator[ContextItemType]:
         """
         Iterates results in document order.
 
@@ -241,12 +253,12 @@ class XPathContext(object):
 
         self.item = status
 
-    def inner_focus_select(self, token):
+    def inner_focus_select(self, token: Union['XPathToken', 'XPathAxis']) -> Iterator[Any]:
         """Apply the token's selector with an inner focus."""
         status = self.item, self.size, self.position
-        results = [x for x in token.select(self.copy())]
+        results = [x for x in token.select(copy(self))]
 
-        if token.label == 'axis' and token.reverse_axis:
+        if token.label == 'axis' and cast('XPathAxis', token).reverse_axis:
             self.size = self.position = len(results)
             for self.item in results:
                 yield self.item
@@ -258,7 +270,8 @@ class XPathContext(object):
 
         self.item, self.size, self.position = status
 
-    def iter_product(self, selectors, varnames=None):
+    def iter_product(self, selectors: Sequence[Callable[[Any], Any]],
+                     varnames: Optional[Sequence[str]] = None) -> Iterator[Any]:
         """
         Iterator for cartesian products of selectors.
 
@@ -280,10 +293,11 @@ class XPathContext(object):
                 iterators[k] = selectors[k](self)
                 k -= 1
             else:
-                try:
-                    self.variables[varnames[k]] = value
-                except (TypeError, IndexError):
-                    pass
+                if varnames is not None:
+                    try:
+                        self.variables[varnames[k]] = value
+                    except (TypeError, IndexError):
+                        pass
 
                 prod[k] = value
                 if k == max_index:
@@ -294,15 +308,17 @@ class XPathContext(object):
     ##
     # Context item iterators for axis
 
-    def iter_self(self):
+    def iter_self(self) -> Iterator[Any]:
         """Iterator for 'self' axis and '.' shortcut."""
         status = self.axis
         self.axis = 'self'
         yield self.item
         self.axis = status
 
-    def iter_attributes(self):
+    def iter_attributes(self) -> Iterator[Union[AttributeNode, TypedAttribute]]:
         """Iterator for 'attribute' axis and '@' shortcut."""
+        status: Any
+
         if isinstance(self.item, (AttributeNode, TypedAttribute)):
             status = self.axis
             self.axis = 'attribute'
@@ -318,18 +334,18 @@ class XPathContext(object):
         if isinstance(self.item, TypedElement):
             self.item = self.item.elem
 
-        elem = self.item
+        elem = cast(ElementNode, self.item)
         if is_schema_node(elem):
             # TODO: for backward compatibility, to be removed in release 3.0.
             for self.item in (AttributeNode(*x) for x in elem.attrib.items()):
                 yield self.item
         else:
-            for self.item in (AttributeNode(*x, elem) for x in elem.attrib.items()):
+            for self.item in (AttributeNode(x[0], x[1], parent=elem) for x in elem.attrib.items()):
                 yield self.item
 
         self.item, self.axis = status
 
-    def iter_children_or_self(self):
+    def iter_children_or_self(self) -> Iterator[Any]:
         """Iterator for 'child' forward axis and '/' step."""
         if self.axis is not None:
             yield self.item
@@ -342,10 +358,14 @@ class XPathContext(object):
             self.item = self.item.elem
 
         if self.item is None:
-            self.item = self.root.getroot() if is_document_node(self.root) else self.root
+            if is_document_node(self.root):
+                document = cast(DocumentNode, self.root)
+                self.item = document.getroot()
+            else:
+                self.item = self.root
             yield self.item
         elif is_etree_element(self.item):
-            elem = self.item
+            elem = cast(ElementNode, self.item)
             if elem.text is not None:
                 self.item = TextNode(elem.text, elem)
                 yield self.item
@@ -359,12 +379,13 @@ class XPathContext(object):
                     yield self.item
 
         elif is_document_node(self.item):
-            self.item = self.item.getroot()
+            document = cast(DocumentNode, self.item)
+            self.item = document.getroot()
             yield self.item
 
         self.item, self.axis = status
 
-    def iter_parent(self):
+    def iter_parent(self) -> Iterator[ElementNode]:
         """Iterator for 'parent' reverse axis and '..' shortcut."""
         if isinstance(self.item, TypedElement):
             parent = self.get_parent(self.item.elem)
@@ -376,22 +397,22 @@ class XPathContext(object):
             self.axis = 'parent'
 
             self.item = parent
-            yield self.item
+            yield cast(ElementNode, self.item)
 
             self.item, self.axis = status
 
-    def iter_siblings(self, axis=None):
+    def iter_siblings(self, axis: Optional[str] = None) -> Iterator[ElementNode]:
         """
         Iterator for 'following-sibling' forward axis and 'preceding-sibling' reverse axis.
 
         :param axis: the context axis, default is 'following-sibling'.
         """
         if isinstance(self.item, TypedElement):
-            item = self.item.elem
-        elif not is_etree_element(self.item) or callable(self.item.tag):
+            item = cast(ElementNode, self.item.elem)
+        elif not is_etree_element(self.item) or callable(getattr(self.item, 'tag')):
             return
         else:
-            item = self.item
+            item = cast(ElementNode, self.item)
 
         parent = self.get_parent(item)
         if parent is None:
@@ -417,7 +438,8 @@ class XPathContext(object):
 
         self.item, self.axis = status
 
-    def iter_descendants(self, axis=None, inner_focus=False):
+    def iter_descendants(self, axis: Optional[str] = None,
+                         inner_focus: bool = False) -> Iterator[Any]:
         """
         Iterator for 'descendant' and 'descendant-or-self' forward axes and '//' shortcut.
 
@@ -455,13 +477,13 @@ class XPathContext(object):
 
             self.item, self.position, self.size, self.axis = status
         else:
-            status = self.item, self.axis
+            status_ = self.item, self.axis
             self.axis = axis
             for self.item in descendants:
                 yield self.item
-            self.item, self.axis = status
+            self.item, self.axis = status_
 
-    def iter_ancestors(self, axis=None):
+    def iter_ancestors(self, axis: Optional[str] = None) -> Iterator[ContextRootType]:
         """
         Iterator for 'ancestor' and 'ancestor-or-self' reverse axes.
 
@@ -471,10 +493,17 @@ class XPathContext(object):
         self.axis = axis or 'ancestor'
 
         if isinstance(self.item, TypedElement):
-            self.item = self.item.elem
+            elem = cast(ElementNode, self.item.elem)
+        elif is_element_node(self.item):
+            elem = cast(ElementNode, self.item)
+        else:
+            return
 
-        ancestors = [self.item] if self.axis == 'ancestor-or-self' else []
-        parent = self.get_parent(self.item)
+        ancestors: List[Union[ElementNode, DocumentNode]] = []
+        if self.axis == 'ancestor-or-self':
+            ancestors.append(elem)
+
+        parent = self.get_parent(elem)
         while parent is not None:
             ancestors.append(parent)
             parent = self.get_parent(parent)
@@ -484,10 +513,11 @@ class XPathContext(object):
 
         self.item, self.axis = status
 
-    def iter_preceding(self):
+    def iter_preceding(self) -> Iterator[ElementNode]:
         """Iterator for 'preceding' reverse axis."""
         item = self.item.elem if isinstance(self.item, TypedElement) else self.item
-        if not is_etree_element(item) or item is self.root or callable(item.tag):
+        if not is_etree_element(item) or item is self.root \
+                or callable(getattr(item, 'tag')):
             return
 
         status = self.item, self.axis
@@ -512,7 +542,7 @@ class XPathContext(object):
 
         self.item, self.axis = status
 
-    def iter_followings(self):
+    def iter_followings(self) -> Iterator[ElementNode]:
         """Iterator for 'following' forward axis."""
         status = self.item, self.axis
         self.axis = 'following'
@@ -521,7 +551,8 @@ class XPathContext(object):
             return
         elif isinstance(self.item, TypedElement):
             self.item = self.item.elem
-        elif not is_etree_element(self.item) or callable(self.item.tag):
+        elif not is_etree_element(self.item) \
+                or callable(getattr(self.item, 'tag')):
             return
 
         descendants = set(self._iter_nodes(self.item))
